package dev.flyfish.framework.beans.meta.parser.chain.impl;

import dev.flyfish.framework.beans.meta.parser.BeanPropertyAnnotations;
import dev.flyfish.framework.beans.meta.parser.chain.AnnotationChainSupport;
import lombok.RequiredArgsConstructor;

/**
 * 基本的链式支持
 *
 * @author wangyu
 */
@RequiredArgsConstructor
abstract class BasicAnnotationChainSupport implements AnnotationChainSupport {

    // 父级
    protected final BeanPropertyAnnotations parent;

    /**
     * 下一步要做什么
     *
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotations and() {
        return cache().last(false);
    }

    /**
     * 下一步要做什么，如果这一步不成立，才执行
     *
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotations or() {
        return cache();
    }

    /**
     * 主动停止操作，会清空前面的状态
     */
    @Override
    public boolean end() {
        boolean result = isPresent();
        parent.last(false).value(null);
        return result;
    }

    /**
     * 缓存值逻辑
     *
     * @return 父亲
     */
    protected abstract BeanPropertyAnnotations cache();
}
