package dev.flyfish.framework.form.domain;

import dev.flyfish.framework.annotations.Entity;
import dev.flyfish.framework.bean.ObjectMap;
import dev.flyfish.framework.domain.base.Domain;
import lombok.Data;
import org.springframework.data.mongodb.core.index.CompoundIndex;

import java.util.ArrayList;
import java.util.List;

/**
 * 在线表单
 */
@Entity(table = "t_online_form", collection = "online")
@CompoundIndex(name = "uniq_code", def = "{code: 1}", unique = true)
@Data
public class OnlineForm extends Domain {

    // 表单描述
    private String description;

    // 表单配置
    private List<FormItem> form = new ArrayList<>();

    // 分组们，包含标识和名称
    private List<FormGroup> groups = new ArrayList<>();

    // 表单样式，支持default， tabs， step
    private String type = "default";

    // 宽度
    private Integer width;

    // 默认布局
    private ObjectMap layout = new ObjectMap();

    @Data
    public static class FormItem {

        // 表单编码，存储数据的key
        private String code;
        // 显示名称，这个名称是输入框的label
        private String title = "";
        // 组件，标识使用的组件
        private String component;
        // 占据栅格，用于表单布局
        private Integer span = 12;
        // 栅格偏移，布局需要
        private Integer offset = 0;
        // 栅格布局
        private ObjectMap grid = new ObjectMap();
        // 儿子，手动指定儿子们
        private String children;
        // 条件，用于匹配条件以显示与否
        private ObjectMap condition;
        // 校验规则，允许多个
        private List<ObjectMap> validation;
        // 选项，用于表单的额外选项，支持antd vue的所有逻辑
        private ObjectMap options = new ObjectMap();
        // 组件属性，支持对象
        private ObjectMap props;
        // 表单联动，key为表单组件的属性，包含两种模式，condition是激活条件，value是激活的值
        private ObjectMap links;
        // 事件支持
        private ObjectMap events;
        // 映射支持
        private ObjectMap mapping;
        // 异步的props
        private ObjectMap delay;
        // 分组
        private String group;
        // 新增，增加反馈状态
        private Boolean feedback = false;
        // 新增，增加额外描述
        private String extra;
        // 新增，额外的复杂帮助，在tooltip弹出
        private String tips;
    }

    /**
     * 表单分组
     */
    @Data
    public static class FormGroup {

        // 分组名称
        private String name;

        // 分组code
        private String code;

        // 分组表单
        private List<FormItem> form;
    }
}
