package dev.flyfish.framework.logging.config;

import org.springframework.beans.factory.InitializingBean;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 日志文本注册器
 *
 * @author wangyu
 */
public class LoggingTextRegistry implements InitializingBean {

    // 映射
    private final Map<String, String> mapping = new HashMap<>();

    // 自定义处理
    private final List<LoggingTextModifier> modifiers;

    public LoggingTextRegistry(List<LoggingTextModifier> modifiers) {
        this.modifiers = modifiers;
        this.mapping.put("CREATE", "新增");
        this.mapping.put("UPDATE", "更新");
        this.mapping.put("UPDATE_ALL", "更新全部");
        this.mapping.put("DELETE", "删除");
        this.mapping.put("SYNC", "同步");
        this.mapping.put("LOGIN", "系统登录");
        this.mapping.put("LOGOUT", "退出登录");
    }

    public String text(String code) {
        return mapping.getOrDefault(code, code);
    }

    @Override
    public void afterPropertiesSet() {
        modifiers.forEach(modifier -> modifier.modify(mapping));
    }
}
