package dev.flyfish.framework.user.config.audit;

import dev.flyfish.framework.auditor.ReactiveBeanAuditor;
import dev.flyfish.framework.auditor.ReactiveBeanPoster;
import dev.flyfish.framework.user.service.UserDetailsConverter;
import dev.flyfish.framework.user.utils.StrengthUtils;
import dev.flyfish.framework.user.config.constants.UserCacheKeys;
import dev.flyfish.framework.domain.po.User;
import dev.flyfish.framework.enums.UserType;
import dev.flyfish.framework.utils.Assert;
import dev.flyfish.framework.utils.ReactiveRedisOperations;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

@Component
@RequiredArgsConstructor
public class UserBeanAuditor implements ReactiveBeanAuditor<User>, ReactiveBeanPoster<User> {

    private final PasswordEncoder passwordEncoder;
    private final ReactiveRedisOperations reactiveRedisOperations;
    private final UserDetailsConverter userDetailsConverter;

    /**
     * 对实体进行审查，并补全相关字段
     *
     * @param entity 原数据
     */
    @Override
    public Mono<User> audit(User entity) {
        if (null == entity.getId() && StringUtils.isNotBlank(entity.getPassword())) {
            Assert.isTrue(StrengthUtils.isValid(entity.getPassword()), "密码强度不够，至少应该包含数字、大小写字母、符号组合");
            entity.setPassword(passwordEncoder.encode(entity.getPassword()));
        }
        if (null == entity.getType()) {
            entity.setType(UserType.ADMIN);
        }
        if (null == entity.getEnable()) {
            entity.setEnable(true);
        }
        if (null == entity.getApp()) {
            entity.setApp(false);
        }
        if (null == entity.getCode()) {
            entity.setCode(entity.getUsername());
        }
        return Mono.just(entity);
    }

    /**
     * 对入库的实体进行审查，并执行额外功能
     *
     * @param data 原数据
     */
    @Override
    public Mono<User> post(User data) {
        // 更新缓存
        return userDetailsConverter.mapToUserDetails(data)
                .flatMap(detail -> reactiveRedisOperations.set(UserCacheKeys.get(data.getUsername()), detail))
                .thenReturn(data);
    }
}
