package dev.flyfish.framework.utils;

import dev.flyfish.framework.domain.base.IUser;
import dev.flyfish.framework.domain.po.User;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.userdetails.UserDetails;

/**
 * 用户工具
 *
 * @author wangyu
 */
public class UserUtils {

    /**
     * 解压用户
     *
     * @param securityContext 上下文
     * @return 结果
     */
    public static User extractUser(SecurityContext securityContext) {
        return ((IUser) securityContext.getAuthentication().getPrincipal()).toUser();
    }

    /**
     * 解压用户详情
     *
     * @param securityContext 上下文
     * @return 结果
     */
    public static IUser extractUserDetails(SecurityContext securityContext) {
        return (IUser) securityContext.getAuthentication().getPrincipal();
    }

    /**
     * 解压用户名称
     *
     * @param securityContext 上下文
     * @return 结果
     */
    public static String extractUserName(SecurityContext securityContext) {
        return ((IUser) securityContext.getAuthentication().getPrincipal()).getName();
    }

    /**
     * 转换用户
     *
     * @param userDetails 用户详情
     * @return 结果
     */
    public static User toUser(UserDetails userDetails) {
        if (null == userDetails) {
            return null;
        }
        if (userDetails instanceof IUser) {
            return ((IUser) userDetails).toUser();
        }
        return null;
    }
}
